@@ -1,8 +1,8 @@
 /*
  * Camera test application
  *
- * Copyright (C) 2016 Renesas Electronics Corporation
- * Copyright (C) 2016 Cogent Embedded, Inc. <source@cogentembedded.com>
+ * Copyright (C) 2016-2017 Renesas Electronics Corporation
+ * Copyright (C) 2016-2017 Cogent Embedded, Inc. <source@cogentembedded.com>
  *
  * based on:
  *  V4L2 video capture example
@@ -45,19 +45,24 @@ struct buffer {
         size_t  length;
 };
 
-static char            *dev_name;
+#define N_DEVS_MAX      8
+static char             n_devs = 1;
+static char            *dev_name[N_DEVS_MAX] = {"/dev/video0","/dev/video1","/dev/video2","/dev/video3","/dev/video4","/dev/video5","/dev/video6","/dev/video7"};
 static char            *fbdev_name;
 static enum io_method   io = IO_METHOD_MMAP;
 //static enum io_method   io = IO_METHOD_USERPTR;
-static int              fd = -1;
+static int              fd[N_DEVS_MAX] = {-1, -1, -1, -1, -1, -1, -1, -1};
 static int              fbfd = -1;
-struct buffer          *buffers;
-static unsigned int     n_buffers;
+struct buffer          *buffers[N_DEVS_MAX];
+static unsigned int     n_buffers[N_DEVS_MAX];
 static int              out_buf, out_fb;
 static char             *format_name;
 static int              frame_count = 70;
 static int              fps_count = 0;
 static int              framerate = 0;
+static int              timeout = 60; // secs
+static int              LEFT = 0;
+static int              TOP = 0;
 static int              WIDTH = 1280;
 static int              HEIGHT = 800;
 static struct fb_var_screeninfo vinfo;
@@ -65,6 +70,7 @@ static struct fb_fix_screeninfo finfo;
 static long int screensize = 0;
 static char *fbmem = 0;
 
+
 static void errno_exit(const char *s)
 {
         fprintf(stderr, "%s error %d, %s\n", s, errno, strerror(errno));
@@ -82,6 +88,29 @@ static int xioctl(int fh, int request, void *arg)
         return r;
 }
 
+static inline unsigned long uSecElapsed(struct timeval *t2, struct timeval *t1)
+{
+        return (t2->tv_sec - t1->tv_sec) * 1000000 + t2->tv_usec - t1->tv_usec;
+}
+
+static void fpsCount(int dev)
+{
+        static unsigned frames[N_DEVS_MAX];
+        static struct timeval frame_time[N_DEVS_MAX];
+        static unsigned long usec[N_DEVS_MAX];
+        struct timeval t;
+
+        gettimeofday(&t, NULL);
+        usec[dev] += frames[dev]++ ? uSecElapsed(&t, &frame_time[dev]) : 0;
+        frame_time[dev] = t;
+        if (usec[dev] >= 1000000) {
+                unsigned fps = ((unsigned long long)frames[dev] * 10000000 + usec[dev] - 1) / usec[dev];
+                fprintf(stderr, "%s FPS: %3u.%1u\n", dev_name[dev], fps / 10, fps % 10);
+                usec[dev] = 0;
+                frames[dev] = 0;
+        }
+}
+
 #if 0
 static inline void yuv_to_rgb32(unsigned char y,
                                 unsigned char u,
@@ -89,7 +118,6 @@ static inline void yuv_to_rgb32(unsigned char y,
                                 unsigned char *rgb)
 {
     register int r,g,b;
-
     r = (1192 * (y - 16) + 1634 * (v - 128) ) >> 10;
     g = (1192 * (y - 16) - 833 * (v - 128) - 400 * (u -128) ) >> 10;
     b = (1192 * (y - 16) + 2066 * (u - 128) ) >> 10;
@@ -105,7 +133,7 @@ static inline void yuv_to_rgb32(unsigned char y,
 }
 #endif
 
-static void process_image(const void *p, int size)
+static void process_image(const void *p, int size, int dev)
 {
         if (out_buf)
                 fwrite(p, size, 1, stdout);
@@ -138,7 +166,8 @@ static void process_image(const void *p, int size)
         /* for RGB32 from camera */
         if (out_fb) {
                 int i;
-                unsigned char *fbp = (unsigned char *)fbmem;
+                int offset = (WIDTH*4)*(dev%(n_devs > 4 ? 4 : 2)) + (HEIGHT*finfo.line_length)*(dev/(n_devs > 4 ? 4 : 2));
+                unsigned char *fbp = (unsigned char *)fbmem + offset;
                 char *buf = (char *)p;
 
                 for (i = 0; i < HEIGHT; i++) {
@@ -148,35 +177,28 @@ static void process_image(const void *p, int size)
                 }
         }
 #endif
-
-//        fflush(stderr);
-//        fprintf(stderr, ".");
-//        fflush(stdout);
 }
 
-static int read_frame(void)
+static int read_frame(int dev)
 {
         struct v4l2_buffer buf;
         unsigned int i;
 
         switch (io) {
         case IO_METHOD_READ:
-                if (-1 == read(fd, buffers[0].start, buffers[0].length)) {
+                if (-1 == read(fd[dev], (buffers[dev])[0].start, (buffers[dev])[0].length)) {
                         switch (errno) {
                         case EAGAIN:
                                 return 0;
-
                         case EIO:
                                 /* Could ignore EIO, see spec. */
-
                                 /* fall through */
-
                         default:
                                 errno_exit("read");
                         }
                 }
 
-                process_image(buffers[0].start, buffers[0].length);
+                process_image((buffers[dev])[0].start, (buffers[dev])[0].length, dev);
                 break;
 
         case IO_METHOD_MMAP:
@@ -185,26 +207,23 @@ static int read_frame(void)
                 buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
                 buf.memory = V4L2_MEMORY_MMAP;
 
-                if (-1 == xioctl(fd, VIDIOC_DQBUF, &buf)) {
+                if (-1 == xioctl(fd[dev], VIDIOC_DQBUF, &buf)) {
                         switch (errno) {
                         case EAGAIN:
                                 return 0;
-
                         case EIO:
                                 /* Could ignore EIO, see spec. */
-
                                 /* fall through */
-
                         default:
                                 errno_exit("VIDIOC_DQBUF");
                         }
                 }
 
-                assert(buf.index < n_buffers);
+                assert(buf.index < n_buffers[dev]);
 
-                process_image(buffers[buf.index].start, buf.bytesused);
+                process_image((buffers[dev])[buf.index].start, buf.bytesused, dev);
 
-                if (-1 == xioctl(fd, VIDIOC_QBUF, &buf))
+                if (-1 == xioctl(fd[dev], VIDIOC_QBUF, &buf))
                         errno_exit("VIDIOC_QBUF");
                 break;
 
@@ -214,82 +233,62 @@ static int read_frame(void)
                 buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
                 buf.memory = V4L2_MEMORY_USERPTR;
 
-                if (-1 == xioctl(fd, VIDIOC_DQBUF, &buf)) {
+                if (-1 == xioctl(fd[dev], VIDIOC_DQBUF, &buf)) {
                         switch (errno) {
                         case EAGAIN:
                                 return 0;
-
                         case EIO:
                                 /* Could ignore EIO, see spec. */
-
                                 /* fall through */
-
                         default:
                                 errno_exit("VIDIOC_DQBUF");
                         }
                 }
 
-                for (i = 0; i < n_buffers; ++i)
-                        if (buf.m.userptr == (unsigned long)buffers[i].start
-                            && buf.length == buffers[i].length)
+                for (i = 0; i < n_buffers[dev]; ++i)
+                        if (buf.m.userptr == (unsigned long)(buffers[dev])[i].start
+                            && buf.length == (buffers[dev])[i].length)
                                 break;
 
-                assert(i < n_buffers);
+                assert(i < n_buffers[dev]);
 
-                process_image((void *)buf.m.userptr, buf.bytesused);
+                process_image((void *)buf.m.userptr, buf.bytesused, dev);
 
-                if (-1 == xioctl(fd, VIDIOC_QBUF, &buf))
+                if (-1 == xioctl(fd[dev], VIDIOC_QBUF, &buf))
                         errno_exit("VIDIOC_QBUF");
                 break;
         }
 
+        if (fps_count)
+                fpsCount(dev);
+
         return 1;
 }
 
-static inline unsigned long uSecElapsed(struct timeval *t2, struct timeval *t1)
-{
-    return (t2->tv_sec - t1->tv_sec) * 1000000 + t2->tv_usec - t1->tv_usec;
-}
-
-static void fpsCount()
-{
-    static unsigned frames;
-    static struct timeval frame_time;
-    static unsigned long usec;
-    struct timeval t;
-
-    gettimeofday(&t, NULL);
-    usec += frames++ ? uSecElapsed(&t, &frame_time) : 0;
-    frame_time = t;
-    if (usec >= 1000000) {
-        unsigned fps = ((unsigned long long)frames * 10000000 + usec - 1) / usec;
-        fprintf(stderr, "%s FPS: %3u.%1u\n", dev_name, fps / 10, fps % 10);
-        usec = 0;
-        frames = 0;
-    }
-}
+#define max(a,b) (a>b?a:b)
 
 static void mainloop(void)
 {
-        unsigned int count;
+        unsigned int count = frame_count;
+        int dev = 0;
+        fd_set fds;
+        struct timeval tv;
+        int r;
 
-        count = frame_count;
+        /* Give time to queue buffers at start streaming by VIN module */
+        usleep(34000*3);
 
         while (count-- > 0) {
                 for (;;) {
-                        fd_set fds;
-                        struct timeval tv;
-                        int r;
-
                         FD_ZERO(&fds);
-                        FD_SET(fd, &fds);
 
+                        for (dev = 0; dev < n_devs; dev++)
+                                FD_SET(fd[dev], &fds);
                         /* Timeout. */
-                        tv.tv_sec = 2;
+                        tv.tv_sec = timeout;
                         tv.tv_usec = 0;
 
-                        r = select(fd + 1, &fds, NULL, NULL, &tv);
-
+                        r = select(max(max(max(fd[0],fd[1]),max(fd[2],fd[3])),max(max(fd[4],fd[5]),max(fd[6],fd[7]))) + 1, &fds, NULL, NULL, &tv);
                         if (-1 == r) {
                                 if (EINTR == errno)
                                         continue;
@@ -301,17 +300,19 @@ static void mainloop(void)
                                 exit(EXIT_FAILURE);
                         }
 
-                        if (read_frame())
+                        r = 0;
+                        for (dev = 0; dev < n_devs; dev++) {
+                                if (FD_ISSET(fd[dev], &fds))
+                                        r += read_frame(dev);
+                        }
+                        if (r)
                                 break;
                         /* EAGAIN - continue select loop. */
                 }
-
-                if (fps_count)
-                            fpsCount();
         }
 }
 
-static void stop_capturing(void)
+static void stop_capturing(int dev)
 {
         enum v4l2_buf_type type;
 
@@ -323,13 +324,13 @@ static void stop_capturing(void)
         case IO_METHOD_MMAP:
         case IO_METHOD_USERPTR:
                 type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-                if (-1 == xioctl(fd, VIDIOC_STREAMOFF, &type))
+                if (-1 == xioctl(fd[dev], VIDIOC_STREAMOFF, &type))
                         errno_exit("VIDIOC_STREAMOFF");
                 break;
         }
 }
 
-static void start_capturing(void)
+static void start_capturing(int dev)
 {
         unsigned int i;
         enum v4l2_buf_type type;
@@ -340,7 +341,7 @@ static void start_capturing(void)
                 break;
 
         case IO_METHOD_MMAP:
-                for (i = 0; i < n_buffers; ++i) {
+                for (i = 0; i < n_buffers[dev]; ++i) {
                         struct v4l2_buffer buf;
 
                         CLEAR(buf);
@@ -348,78 +349,78 @@ static void start_capturing(void)
                         buf.memory = V4L2_MEMORY_MMAP;
                         buf.index = i;
 
-                        if (-1 == xioctl(fd, VIDIOC_QBUF, &buf))
+                        if (-1 == xioctl(fd[dev], VIDIOC_QBUF, &buf))
                                 errno_exit("VIDIOC_QBUF");
                 }
                 type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-                if (-1 == xioctl(fd, VIDIOC_STREAMON, &type))
+                if (-1 == xioctl(fd[dev], VIDIOC_STREAMON, &type))
                         errno_exit("VIDIOC_STREAMON");
                 break;
 
         case IO_METHOD_USERPTR:
-                for (i = 0; i < n_buffers; ++i) {
+                for (i = 0; i < n_buffers[dev]; ++i) {
                         struct v4l2_buffer buf;
 
                         CLEAR(buf);
                         buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
                         buf.memory = V4L2_MEMORY_USERPTR;
                         buf.index = i;
-                        buf.m.userptr = (unsigned long)buffers[i].start;
-                        buf.length = buffers[i].length;
+                        buf.m.userptr = (unsigned long)(buffers[dev])[i].start;
+                        buf.length = (buffers[dev])[i].length;
 
-                        if (-1 == xioctl(fd, VIDIOC_QBUF, &buf))
+                        if (-1 == xioctl(fd[dev], VIDIOC_QBUF, &buf))
                                 errno_exit("VIDIOC_QBUF");
                 }
                 type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-                if (-1 == xioctl(fd, VIDIOC_STREAMON, &type))
+                if (-1 == xioctl(fd[dev], VIDIOC_STREAMON, &type))
                         errno_exit("VIDIOC_STREAMON");
                 break;
         }
 }
 
-static void uninit_device(void)
+static void uninit_device(int dev)
 {
         unsigned int i;
 
         switch (io) {
         case IO_METHOD_READ:
-                free(buffers[0].start);
+                free((buffers[dev])[0].start);
                 break;
 
         case IO_METHOD_MMAP:
-                for (i = 0; i < n_buffers; ++i)
-                        if (-1 == munmap(buffers[i].start, buffers[i].length))
+                for (i = 0; i < n_buffers[dev]; ++i)
+                        if (-1 == munmap((buffers[dev])[i].start, (buffers[dev])[i].length))
                                 errno_exit("munmap");
                 break;
 
         case IO_METHOD_USERPTR:
-                for (i = 0; i < n_buffers; ++i)
-                        free(buffers[i].start);
+                for (i = 0; i < n_buffers[dev]; ++i)
+                        free((buffers[dev])[i].start);
                 break;
         }
 
-        free(buffers);
+        free(buffers[dev]);
 }
 
-static void init_read(unsigned int buffer_size)
+static void init_read(unsigned int buffer_size, int dev)
 {
-        buffers = calloc(1, sizeof(*buffers));
+        buffers[dev] = calloc(1, sizeof(*buffers[dev]));
 
-        if (!buffers) {
+        if (!buffers[dev]) {
                 fprintf(stderr, "Out of memory\n");
                 exit(EXIT_FAILURE);
         }
 
-        buffers[0].length = buffer_size;
-        buffers[0].start = malloc(buffer_size);
+        (buffers[dev])[0].length = buffer_size;
+        (buffers[dev])[0].start = malloc(buffer_size);
 
-        if (!buffers[0].start) {
+        if (!(buffers[dev])[0].start) {
                 fprintf(stderr, "Out of memory\n");
                 exit(EXIT_FAILURE);
         }
 }
 
-static void init_mmap(void)
+static void init_mmap(int dev)
 {
         struct v4l2_requestbuffers req;
 
@@ -429,10 +430,10 @@ static void init_mmap(void)
         req.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
         req.memory = V4L2_MEMORY_MMAP;
 
-        if (-1 == xioctl(fd, VIDIOC_REQBUFS, &req)) {
+        if (-1 == xioctl(fd[dev], VIDIOC_REQBUFS, &req)) {
                 if (EINVAL == errno) {
                         fprintf(stderr, "%s does not support "
-                                 "memory mapping\n", dev_name);
+                                 "memory mapping\n", dev_name[dev]);
                         exit(EXIT_FAILURE);
                 } else {
                         errno_exit("VIDIOC_REQBUFS");
@@ -441,43 +442,43 @@ static void init_mmap(void)
 
         if (req.count < 2) {
                 fprintf(stderr, "Insufficient buffer memory on %s\n",
-                         dev_name);
+                         dev_name[dev]);
                 exit(EXIT_FAILURE);
         }
 
-        buffers = calloc(req.count, sizeof(*buffers));
+        buffers[dev] = calloc(req.count, sizeof(*buffers[dev]));
 
-        if (!buffers) {
+        if (!buffers[dev]) {
                 fprintf(stderr, "Out of memory\n");
                 exit(EXIT_FAILURE);
         }
 
-        for (n_buffers = 0; n_buffers < req.count; ++n_buffers) {
+        for (n_buffers[dev] = 0; n_buffers[dev] < req.count; ++n_buffers[dev]) {
                 struct v4l2_buffer buf;
 
                 CLEAR(buf);
 
                 buf.type        = V4L2_BUF_TYPE_VIDEO_CAPTURE;
                 buf.memory      = V4L2_MEMORY_MMAP;
-                buf.index       = n_buffers;
+                buf.index       = n_buffers[dev];
 
-                if (-1 == xioctl(fd, VIDIOC_QUERYBUF, &buf))
+                if (-1 == xioctl(fd[dev], VIDIOC_QUERYBUF, &buf))
                         errno_exit("VIDIOC_QUERYBUF");
 
-                buffers[n_buffers].length = buf.length;
-                buffers[n_buffers].start =
+                (buffers[dev])[n_buffers[dev]].length = buf.length;
+                (buffers[dev])[n_buffers[dev]].start =
                         mmap(NULL /* start anywhere */,
                               buf.length,
                               PROT_READ | PROT_WRITE /* required */,
                               MAP_SHARED /* recommended */,
-                              fd, buf.m.offset);
+                              fd[dev], buf.m.offset);
 
-                if (MAP_FAILED == buffers[n_buffers].start)
+                if (MAP_FAILED == (buffers[dev])[n_buffers[dev]].start)
                         errno_exit("mmap");
         }
 }
 
-static void init_userp(unsigned int buffer_size)
+static void init_userp(unsigned int buffer_size, int dev)
 {
         struct v4l2_requestbuffers req;
 
@@ -487,46 +488,45 @@ static void init_userp(unsigned int buffer_size)
         req.type   = V4L2_BUF_TYPE_VIDEO_CAPTURE;
         req.memory = V4L2_MEMORY_USERPTR;
 
-        if (-1 == xioctl(fd, VIDIOC_REQBUFS, &req)) {
+        if (-1 == xioctl(fd[dev], VIDIOC_REQBUFS, &req)) {
                 if (EINVAL == errno) {
                         fprintf(stderr, "%s does not support "
-                                 "user pointer i/o\n", dev_name);
+                                 "user pointer i/o\n", dev_name[dev]);
                         exit(EXIT_FAILURE);
                 } else {
                         errno_exit("VIDIOC_REQBUFS");
                 }
         }
 
-        buffers = calloc(4, sizeof(*buffers));
+        buffers[dev] = calloc(4, sizeof(*buffers[dev]));
 
-        if (!buffers) {
+        if (!buffers[dev]) {
                 fprintf(stderr, "Out of memory\n");
                 exit(EXIT_FAILURE);
         }
 
-        for (n_buffers = 0; n_buffers < 4; ++n_buffers) {
-                buffers[n_buffers].length = buffer_size;
-                buffers[n_buffers].start = malloc(buffer_size);
+        for (n_buffers[dev] = 0; n_buffers[dev] < 4; ++n_buffers[dev]) {
+                (buffers[dev])[n_buffers[dev]].length = buffer_size;
+                (buffers[dev])[n_buffers[dev]].start = malloc(buffer_size);
 
-                if (!buffers[n_buffers].start) {
+                if (!(buffers[dev])[n_buffers[dev]].start) {
                         fprintf(stderr, "Out of memory\n");
                         exit(EXIT_FAILURE);
                 }
         }
 }
 
-static void init_device(void)
+static void init_device(int dev)
 {
         struct v4l2_capability cap;
         struct v4l2_cropcap cropcap;
         struct v4l2_crop crop;
         struct v4l2_format fmt;
-        unsigned int min;
 
-        if (-1 == xioctl(fd, VIDIOC_QUERYCAP, &cap)) {
+        if (-1 == xioctl(fd[dev], VIDIOC_QUERYCAP, &cap)) {
                 if (EINVAL == errno) {
                         fprintf(stderr, "%s is no V4L2 device\n",
-                                 dev_name);
+                                 dev_name[dev]);
                         exit(EXIT_FAILURE);
                 } else {
                         errno_exit("VIDIOC_QUERYCAP");
@@ -535,7 +535,7 @@ static void init_device(void)
 
         if (!(cap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
                 fprintf(stderr, "%s is no video capture device\n",
-                         dev_name);
+                         dev_name[dev]);
                 exit(EXIT_FAILURE);
         }
 
@@ -543,7 +543,7 @@ static void init_device(void)
         case IO_METHOD_READ:
                 if (!(cap.capabilities & V4L2_CAP_READWRITE)) {
                         fprintf(stderr, "%s does not support read i/o\n",
-                                 dev_name);
+                                 dev_name[dev]);
                         exit(EXIT_FAILURE);
                 }
                 break;
@@ -552,7 +552,7 @@ static void init_device(void)
         case IO_METHOD_USERPTR:
                 if (!(cap.capabilities & V4L2_CAP_STREAMING)) {
                         fprintf(stderr, "%s does not support streaming i/o\n",
-                                 dev_name);
+                                 dev_name[dev]);
                         exit(EXIT_FAILURE);
                 }
                 break;
@@ -562,11 +562,16 @@ static void init_device(void)
         /* Select video input, video standard and tune here. */
         CLEAR(cropcap);
         cropcap.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-        if (0 == xioctl(fd, VIDIOC_CROPCAP, &cropcap)) {
+        if (0 == xioctl(fd[dev], VIDIOC_CROPCAP, &cropcap)) {
                 crop.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
                 crop.c = cropcap.defrect; /* reset to default */
 
-                if (-1 == xioctl(fd, VIDIOC_S_CROP, &crop)) {
+                crop.c.left = LEFT;
+                crop.c.top = TOP;
+                crop.c.width = WIDTH;
+                crop.c.height = HEIGHT;
+
+                if (-1 == xioctl(fd[dev], VIDIOC_S_CROP, &crop)) {
                         switch (errno) {
                         case EINVAL:
                                 /* Cropping not supported. */
@@ -584,12 +589,12 @@ static void init_device(void)
             struct v4l2_streamparm parm;
 
             parm.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
-            if (-1 == xioctl(fd, VIDIOC_G_PARM, &parm))
+            if (-1 == xioctl(fd[dev], VIDIOC_G_PARM, &parm))
                 errno_exit("VIDIOC_G_PARM");
 
             parm.parm.capture.timeperframe.numerator = 1;
             parm.parm.capture.timeperframe.denominator = framerate;
-            if (-1 == xioctl(fd, VIDIOC_S_PARM, &parm))
+            if (-1 == xioctl(fd[dev], VIDIOC_S_PARM, &parm))
                 errno_exit("VIDIOC_S_PARM");
         }
 
@@ -606,12 +611,12 @@ static void init_device(void)
 //    control.id = V4L2_CID_EXPOSURE;
 //    control.id = V4L2_CID_AUTOGAIN;
 
-    if (-1 == xioctl(fd, VIDIOC_G_CTRL, &control))
+    if (-1 == xioctl(fd[dev], VIDIOC_G_CTRL, &control))
         errno_exit("VIDIOC_G_CTRL");
 
     control.value = 0xf0;
 
-    if (-1 == ioctl(fd, VIDIOC_S_CTRL, &control))
+    if (-1 == ioctl(fd[dev], VIDIOC_S_CTRL, &control))
         errno_exit("VIDIOC_S_CTRL");
 #endif
 
@@ -626,79 +631,78 @@ static void init_device(void)
                 fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_UYVY;
         else if (!strncmp(format_name, "yuyv", 4))
                 fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_YUYV;
-        else if (!strncmp(format_name, "rgb565", 5))
+        else if (!strncmp(format_name, "rgb565", 6))
                 fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_RGB565;
         else if (!strncmp(format_name, "rgb32", 5))
                 fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_XBGR32;
+        else if (!strncmp(format_name, "nv12", 4))
+                fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_NV12;
+        else if (!strncmp(format_name, "nv16", 4))
+                fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_NV16;
+        else if (!strncmp(format_name, "bggr8", 5))
+                fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_SBGGR8;
+        else if (!strncmp(format_name, "grey", 4))
+                fmt.fmt.pix.pixelformat = V4L2_PIX_FMT_GREY;
         else {
                 /* Preserve original settings as set by v4l2-ctl for example */
-                if (-1 == xioctl(fd, VIDIOC_G_FMT, &fmt))
+                if (-1 == xioctl(fd[dev], VIDIOC_G_FMT, &fmt))
                         errno_exit("VIDIOC_G_FMT");
         }
 
-        if (-1 == xioctl(fd, VIDIOC_S_FMT, &fmt))
+        if (-1 == xioctl(fd[dev], VIDIOC_S_FMT, &fmt))
                 errno_exit("VIDIOC_S_FMT");
 
-        /* Buggy driver paranoia. */
-        min = fmt.fmt.pix.width * 2;
-        if (fmt.fmt.pix.bytesperline < min)
-                fmt.fmt.pix.bytesperline = min;
-        min = fmt.fmt.pix.bytesperline * fmt.fmt.pix.height;
-        if (fmt.fmt.pix.sizeimage < min)
-                fmt.fmt.pix.sizeimage = min;
+//        printf("fmt.fmt.pix.bytesperline =%d\n\n\n",fmt.fmt.pix.bytesperline);
 
         switch (io) {
         case IO_METHOD_READ:
-                init_read(fmt.fmt.pix.sizeimage);
+                init_read(fmt.fmt.pix.sizeimage, dev);
                 break;
 
         case IO_METHOD_MMAP:
-                init_mmap();
+                init_mmap(dev);
                 break;
 
         case IO_METHOD_USERPTR:
-                init_userp(fmt.fmt.pix.sizeimage);
+                init_userp(fmt.fmt.pix.sizeimage, dev);
                 break;
         }
 }
 
-static void close_device(void)
+static void close_device(int dev)
 {
-        if (out_fb) {
-                munmap(fbmem, screensize);
-                close(fbfd);
-                fbfd = -1;
-        }
-
-        if (-1 == close(fd))
+        if (-1 == close(fd[dev]))
                 errno_exit("close");
 
-        fd = -1;
+        fd[dev] = -1;
 }
 
-static void open_device(void)
+static void open_device(int dev)
 {
         struct stat st;
 
-        if (-1 == stat(dev_name, &st)) {
+        if (-1 == stat(dev_name[dev], &st)) {
                 fprintf(stderr, "Cannot identify '%s': %d, %s\n",
-                         dev_name, errno, strerror(errno));
+                         dev_name[dev], errno, strerror(errno));
                 exit(EXIT_FAILURE);
         }
 
         if (!S_ISCHR(st.st_mode)) {
-                fprintf(stderr, "%s is no device\n", dev_name);
+                fprintf(stderr, "%s is no device\n", dev_name[dev]);
                 exit(EXIT_FAILURE);
         }
 
-        fd = open(dev_name, O_RDWR /* required */ | O_NONBLOCK, 0);
+        fd[dev] = open(dev_name[dev], O_RDWR /* required */ | O_NONBLOCK, 0);
 
-        if (-1 == fd) {
+        if (-1 == fd[dev]) {
                 fprintf(stderr, "Cannot open '%s': %d, %s\n",
-                         dev_name, errno, strerror(errno));
+                         dev_name[dev], errno, strerror(errno));
                 exit(EXIT_FAILURE);
         }
+}
 
+static void open_fb(void)
+{
         if (out_fb) {
                 fbfd = open(fbdev_name, O_RDWR, 0);
                 if (-1 == fbfd) {
@@ -745,6 +749,15 @@ static void open_device(void)
         }
 }
 
+static void close_fb(void)
+{
+        if (out_fb) {
+                munmap(fbmem, screensize);
+                close(fbfd);
+                fbfd = -1;
+        }
+}
+
 static void usage(FILE *fp, char **argv)
 {
         fprintf(fp,
@@ -752,27 +765,32 @@ static void usage(FILE *fp, char **argv)
                  "Version 1.3\n"
                  "Options:\n"
                  "-d | --device name   Video device name [%s]\n"
+                 "-D | --ndev          Number of devices to capture simultaneously [%d]\n"
                  "-h | --help          Print this message\n"
                  "-m | --mmap          Use memory mapped buffers [default]\n"
                  "-r | --read          Use read() calls\n"
                  "-u | --userp         Use application allocated buffers\n"
                  "-o | --output        Outputs stream to stdout\n"
                  "-F | --output_fb     Outputs stream to framebuffer\n"
-                 "-f | --format        Set pixel format: uyvy, yuyv, rgb565, rgb32 [%s]\n"
+                 "-f | --format        Set pixel format: uyvy, yuyv, rgb565, rgb32, nv12, nv16, bggr8, grey [%s]\n"
                  "-c | --count         Number of frames to grab [%i]\n"
                  "-z | --fps_count     Enable fps show\n"
                  "-s | --framerate     Set framerate\n"
+                 "-L | --left          Video left crop [%i]\n"
+                 "-T | --top           Video top crop [%i]\n"
                  "-W | --width         Video width [%i]\n"
                  "-H | --height        Video height [%i]\n"
+                 "-t | --timeout       Select timeout [%i]sec\n"
                  "",
-                 argv[0], dev_name, format_name, frame_count, WIDTH, HEIGHT);
+                 argv[0], dev_name[0], n_devs, format_name, frame_count, LEFT, TOP, WIDTH, HEIGHT, timeout);
 }
 
-static const char short_options[] = "d:hmruoFf:c:zs:W:H:";
+static const char short_options[] = "d:D:hmruoFf:c:zs:L:T:W:H:t:";
 
 static const struct option
 long_options[] = {
         { "device", required_argument, NULL, 'd' },
+        { "ndev",   required_argument, NULL, 'D' },
         { "help",   no_argument,       NULL, 'h' },
         { "mmap",   no_argument,       NULL, 'm' },
         { "read",   no_argument,       NULL, 'r' },
@@ -783,14 +801,18 @@ long_options[] = {
         { "count",  required_argument, NULL, 'c' },
         { "fps_count",  required_argument, NULL, 'z' },
         { "framerate",  required_argument, NULL, 's' },
+        { "left",  required_argument, NULL, 'L' },
+        { "top",  required_argument, NULL, 'T' },
         { "width",  required_argument, NULL, 'W' },
         { "height",  required_argument, NULL, 'H' },
+        { "timeout",  required_argument, NULL, 't' },
         { 0, 0, 0, 0 }
 };
 
 int main(int argc, char **argv)
 {
-        dev_name = "/dev/video0";
+        int dev;
+        dev_name[0] = "/dev/video0";
         fbdev_name = "/dev/fb0";
         format_name = "uyvy";
 
@@ -809,7 +831,14 @@ int main(int argc, char **argv)
                         break;
 
                 case 'd':
-                        dev_name = optarg;
+                        dev_name[0] = optarg;
+                        break;
+
+                case 'D':
+                        errno = 0;
+                        n_devs = strtol(optarg, NULL, 0);
+                        if (errno)
+                                errno_exit(optarg);
                         break;
 
                 case 'h':
@@ -858,6 +887,20 @@ int main(int argc, char **argv)
                                 errno_exit(optarg);
                         break;
 
+                case 'L':
+                        errno = 0;
+                        LEFT = strtol(optarg, NULL, 0);
+                        if (errno)
+                                errno_exit(optarg);
+                        break;
+
+                case 'T':
+                        errno = 0;
+                        TOP = strtol(optarg, NULL, 0);
+                        if (errno)
+                                errno_exit(optarg);
+                        break;
+
                 case 'W':
                         errno = 0;
                         WIDTH = strtol(optarg, NULL, 0);
@@ -872,19 +915,32 @@ int main(int argc, char **argv)
                                 errno_exit(optarg);
                         break;
 
+                case 't':
+                        errno = 0;
+                        timeout = strtol(optarg, NULL, 0);
+                        if (errno)
+                                errno_exit(optarg);
+                        break;
+
                 default:
                         usage(stderr, argv);
                         exit(EXIT_FAILURE);
                 }
         }
 
-        open_device();
-        init_device();
-        start_capturing();
+        for (dev = 0; dev < n_devs; dev++) {
+                open_device(dev);
+                init_device(dev);
+                start_capturing(dev);
+        }
+        open_fb();
         mainloop();
-        stop_capturing();
-        uninit_device();
-        close_device();
-//        fprintf(stderr, "\n");
+        close_fb();
+        for (dev = 0; dev < n_devs; dev++) {
+                stop_capturing(dev);
+                uninit_device(dev);
+                close_device(dev);
+        }
+
         return 0;
 }
